#ifndef INC_MilDebugNew_H
#define INC_MilDebugNew_H
//-----------------------------------------------------------------------------
/*!
\file MilDebugNew.h
\brief
\if english
  Header for overloaded new and delete operator.
\endif

\if german
  Header für überladenen new- und delete-operator.
\endif
\author Helmut Jakoby
\copyright © 2021 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the MemoryLeakIndicator module.

MemoryLeakIndicator is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include <new>
//-----------------------------------------------------------------------------
/*!
\if english
  An object is instantiated and registered in MemSpy::LeakMap if MemSpy::LeakMap
  is instantiated.
  param[in] size The size of the object to instantiate.
  param[in] fileName The file name where 'new' was called.
  param[in] line The line number where 'new' was called.
  \return The newly instantiated object.
\endif

\if german
  Es wird ein Objekt instantiiert und in der MemSpy::LeakMap registriert, wenn
  MemSpy::LeakMap instantiiert ist.
  param[in] size Die Größe des zu instantiierenden Objekts.
  param[in] fileName Der Dateiname, in dem 'new' aufgerufen wurde.
  param[in] line Die Zeilennummer, in der 'new' aufgerufen wurde.
  \return Das neu instantiierte Objekt.
\endif
*/
void * debug_new( std::size_t size, const char * fileName, int line );
//-----------------------------------------------------------------------------
/*!
\if english
  This operator new registers a new object in MemSpy::LeakMap via function
  debug_new (...) when MemSpy::LeakMap is instantiated.
  param[in] size The size of the object to instantiate.
  param[in] fileName The file name where 'new' was called.
  param[in] line The line number where 'new' was called.
  \return The newly instantiated object.
\endif

\if german
  Dieser operator new lässt über Funktion debug_new (...) ein neues Objekt in
  der MemSpy::LeakMap registrieren, wenn MemSpy::LeakMap instantiiert ist.
  param[in] size Die Größe des zu instantiierenden Objekts.
  param[in] fileName Der Dateiname, in dem 'new' aufgerufen wurde.
  param[in] line Die Zeilennummer, in der 'new' aufgerufen wurde.
  \return Das neu instantiierte Objekt.
\endif
*/
void * operator new( std::size_t size, const char * fileName, int line );
void * operator new[]( std::size_t size, const char * fileName, int line );
//-----------------------------------------------------------------------------
/*!
\if english
  Passed object will be deregistered in MemSpy::LeakMap when MemSpy::LeakMap
  instantiates and then removed from memory.
  \param[in] ptr Pointer to the object to be deleted.
\endif

\if german
  Übergebenes Objekt wird in der MemSpy::LeakMap deregistriert, wenn MemSpy::LeakMap
  instantiiert und dann aus dem Speicher entfernt.
  \param[in] ptr Zeiger auf das zu löschende Objekt.
\endif
*/
void debug_delete( void * ptr );
//-----------------------------------------------------------------------------
/*!
\if english
  This operator delete deletes and deregisters an object in MemSpy::LeakMap when
  MemSpy::LeakMap instantiates.
  \param[in] ptr Pointer to the object to be deleted.
\endif

\if german
  Dieser operator delete löscht und deregistriert ein Objekt in der MemSpy::LeakMap,
  wenn MemSpy::LeakMap instantiiert.
  \param[in] ptr Zeiger auf das zu löschende Objekt.
\endif
*/
void operator delete( void * ptr );
//-----------------------------------------------------------------------------
/*!
\if english
  This operator delete is not called, but is intended to suppress the following
  warning:
   warning C4291: 'void * operator new(size_t,const char *,int)': no matching operator delete found; memory will not be freed if initialization throws an exception
\endif

\if german
  Dieser operator delete wird nicht aufgerufen, sondern soll folgende Warnung
  unterdrücken:
  warning C4291: "void * operator new(size_t,const char *,int)": Kein zugehöriger delete-Operator gefunden; Speicher wird nicht freigegeben, wenn die Initialisierung eine Ausnahme auslöst
\endif
*/
void operator delete( void *, const char *, int );
//-----------------------------------------------------------------------------
///@{
/**
\if english
  Redefinition of the operator new.
\endif

\if german
  Neudefinition vom operator new.
\endif
*/
#define DEBUG_NEW new( __FILE__, __LINE__ )
#define new DEBUG_NEW
///@}
//-----------------------------------------------------------------------------
#endif
