//-----------------------------------------------------------------------------
/*!
\file MilNewDelete.cpp
\brief Source for overloaded new and delete operator.
\author Helmut Jakoby
\copyright © 2021 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the MemoryLeakIndicator module.

MemoryLeakIndicator is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "MemoryLeakIndicator.h"
//-----------------------------------------------------------------------------
void * debug_new( std::size_t size, const char * fileName, int line )
{
	void * t_pNewHeapObject = std::malloc( size );
  
  if ( t_pNewHeapObject )
	{
    if ( MemSpy::LeakMap::getGlobalLeakMapInstance() )
    {
      MemSpy::LeakMap  * t_pLeakMap = MemSpy::LeakMap::getGlobalLeakMapInstance();
      if ( t_pLeakMap && t_pLeakMap->leakMapUseAllowed() )
      {
        t_pLeakMap->insertSpyObject( t_pNewHeapObject, fileName, line );
      }
    }
    return t_pNewHeapObject;
	}
  throw std::bad_alloc();
};
//-----------------------------------------------------------------------------
void * operator new( std::size_t size, const char * fileName, int line )
{
	return debug_new( size, fileName, line );
};
//-----------------------------------------------------------------------------
void * operator new[]( std::size_t size, const char * fileName, int line )
{
	return debug_new( size, fileName, line );
};
//-----------------------------------------------------------------------------
void debug_delete( void * ptr )
{
  if ( MemSpy::LeakMap::getGlobalLeakMapInstance() )
  {
    MemSpy::LeakMap  * t_pLeakMap = MemSpy::LeakMap::getGlobalLeakMapInstance();
    if ( t_pLeakMap && t_pLeakMap->leakMapUseAllowed() )
    {
      t_pLeakMap->removeSpyObject( ptr );
    }
  }
  std::free( ptr );
  ptr = NULL_PTR;
}
//-----------------------------------------------------------------------------
void operator delete( void * ptr )
{
  debug_delete( ptr );
}
//-----------------------------------------------------------------------------
void operator delete[]( void * ptr )
{
  debug_delete( ptr );
}
//-----------------------------------------------------------------------------
void operator delete( void * ptr, const char *, int )
{
  debug_delete( ptr );
}
//-----------------------------------------------------------------------------
void operator delete[]( void * ptr, const char *, int )
{
  debug_delete( ptr );
}
//-----------------------------------------------------------------------------

